/*
 * sampling.h
 *
 * Copyright (C) 1989, 1991, Craig E. Kolb
 * All rights reserved.
 *
 * This software may be freely copied, modified, and redistributed
 * provided that this copyright notice is preserved on all copies.
 *
 * You may not distribute this software, in whole or in part, as part of
 * any commercial product without the express consent of the authors.
 *
 * There is no warranty or other guarantee of fitness of this software
 * for any purpose.  It is provided solely "as is".
 *
 * $Id$
 *
 * $Log$
 */
#ifndef SAMPLING_H
#define SAMPLING_H

typedef struct SampleInfo {
	int	totsamples,	/* # of samples/pixel */
		sidesamples;	/* sqrt(samples) */
	Float	weight,		/* 1. / samples */
		spacing;	/* 1. / (samples + 1) */
} SampleInfo;

extern SampleInfo Sampling;

extern void	SamplingSetOptions(), UnitCirclePoint();

#endif /* SAMPLING */
